@@ -11,12 +11,15 @@ use crate::state_tree::StateTree;
 use crate::Kernel;
 
 pub mod backtrace;
+
 pub use backtrace::Backtrace;
 
 mod default;
 
 pub use default::DefaultCallManager;
 
+use crate::trace::ExecutionTrace;
+
 /// BlockID representing nil parameters or return data.
 pub const NO_DATA_BLOCK_ID: u32 = 0;
 
@@ -59,8 +62,8 @@ pub trait CallManager: 'static {
         f: impl FnOnce(&mut Self) -> Result<InvocationResult>,
     ) -> Result<InvocationResult>;
 
-    /// Finishes execution, returning the gas used and the machine.
-    fn finish(self) -> (i64, backtrace::Backtrace, Self::Machine);
+    /// Finishes execution, returning the gas used, machine, and exec trace if requested.
+    fn finish(self) -> (FinishRet, Self::Machine);
 
     /// Returns a reference to the machine.
     fn machine(&self) -> &Self::Machine;
@@ -119,6 +122,7 @@ pub trait CallManager: 'static {
 }
 
 /// The result of a method invocation.
+#[derive(Clone, Debug)]
 pub enum InvocationResult {
     /// Indicates that the actor successfully returned. The value may be empty.
     Return(RawBytes),
@@ -142,3 +146,10 @@ impl InvocationResult {
         }
     }
 }
+
+/// The returned values upon finishing a call manager.
+pub struct FinishRet {
+    pub gas_used: i64,
+    pub backtrace: Backtrace,
+    pub exec_trace: ExecutionTrace,
+}
