@@ -32,10 +32,6 @@ lazy_static! {
         send_transfer_only_premium: 159672,
         send_invoke_method: -5377,
 
-        ipld_get_base: 114617,
-        ipld_put_base: 353640,
-        ipld_put_per_byte: 1,
-
         create_actor_compute: 1108454,
         create_actor_storage: 36 + 40,
         delete_actor: -(36 + 40),
@@ -121,7 +117,146 @@ lazy_static! {
         .iter()
         .copied()
         .collect(),
-        verify_post_discount: false,
+
+        gas_per_exec_unit: 0,
+        get_randomness_base: 0,
+        get_randomness_per_byte: 0,
+
+        block_memcpy_per_byte_cost: 0,
+        block_io_per_byte_cost: 0,
+        block_link_per_byte_cost: 1,
+
+        block_open_base: 114617,
+        block_read_base: 0,
+        block_create_base: 0,
+        block_link_base: 353640,
+        block_stat: 0,
+    };
+
+    static ref SKYR_PRICES: PriceList = PriceList {
+        compute_gas_multiplier: 1,
+        storage_gas_multiplier: 1300,
+
+        on_chain_message_compute_base: 38863,
+        on_chain_message_storage_base: 36,
+        on_chain_message_storage_per_byte: 1,
+
+        on_chain_return_value_per_byte: 1,
+
+        send_base: 29233,
+        send_transfer_funds: 27500,
+        send_transfer_only_premium: 159672,
+        send_invoke_method: -5377,
+
+        create_actor_compute: 1108454,
+        create_actor_storage: 36 + 40,
+        delete_actor: -(36 + 40),
+
+        bls_sig_cost: 16598605,
+        secp256k1_sig_cost: 1637292,
+
+        hashing_base: 31355,
+        compute_unsealed_sector_cid_base: 98647,
+        verify_seal_base: 2000, // TODO revisit potential removal of this
+
+        verify_aggregate_seal_base: 0,
+        verify_aggregate_seal_per: [
+            (
+                RegisteredSealProof::StackedDRG32GiBV1P1,
+                449900
+            ),
+            (
+                RegisteredSealProof::StackedDRG64GiBV1P1,
+                359272
+            )
+        ].iter().copied().collect(),
+        verify_aggregate_seal_steps: [
+            (
+                RegisteredSealProof::StackedDRG32GiBV1P1,
+                StepCost (
+                    vec![
+                        Step{start: 4, cost: 103994170},
+                        Step{start: 7, cost: 112356810},
+                        Step{start: 13, cost: 122912610},
+                        Step{start: 26, cost: 137559930},
+                        Step{start: 52, cost: 162039100},
+                        Step{start: 103, cost: 210960780},
+                        Step{start: 205, cost: 318351180},
+                        Step{start: 410, cost: 528274980},
+                    ]
+                )
+            ),
+            (
+                RegisteredSealProof::StackedDRG64GiBV1P1,
+                StepCost (
+                    vec![
+                        Step{start: 4, cost: 102581240},
+                        Step{start: 7, cost: 110803030},
+                        Step{start: 13, cost: 120803700},
+                        Step{start: 26, cost: 134642130},
+                        Step{start: 52, cost: 157357890},
+                        Step{start: 103, cost: 203017690},
+                        Step{start: 205, cost: 304253590},
+                        Step{start: 410, cost: 509880640},
+                    ]
+                )
+            )
+        ].iter()
+        .cloned()
+        .collect(),
+
+        // TODO: PARAM_FINISH: this may need to be increased to account for the cost of an extern
+        verify_consensus_fault: 495422,
+        verify_replica_update: 36316136,
+        verify_post_lookup: [
+            (
+                RegisteredPoStProof::StackedDRGWindow512MiBV1,
+                ScalingCost {
+                    flat: 117680921,
+                    scale: 43780,
+                },
+            ),
+            (
+                RegisteredPoStProof::StackedDRGWindow32GiBV1,
+                ScalingCost {
+                    flat: 117680921,
+                    scale: 43780,
+                },
+            ),
+            (
+                RegisteredPoStProof::StackedDRGWindow64GiBV1,
+                ScalingCost {
+                    flat: 117680921,
+                    scale: 43780,
+                },
+            ),
+        ]
+        .iter()
+        .copied()
+        .collect(),
+
+        block_memcpy_per_byte_cost: 4,
+        block_io_per_byte_cost: 2,
+        block_link_per_byte_cost: 1,
+        // TODO: PARAM_FINISH
+
+        // TODO: PARAM_FINISH
+        gas_per_exec_unit: 2,
+        // TODO: PARAM_FINISH
+        get_randomness_base: 1,
+        // TODO: PARAM_FINISH
+        get_randomness_per_byte: 1,
+
+        // TODO: PARAM_FINIuiSH
+        block_open_base: 1,
+        // TODO: PARAM_FINISH
+        block_read_base: 1,
+        // TODO: PARAM_FINISH
+        block_create_base: 1,
+        // TODO: PARAM_FINISH
+        block_link_base: 1,
+        // TODO: PARAM_FINISH
+        block_stat: 1,
     };
 }
 
@@ -204,18 +339,6 @@ pub struct PriceList {
     /// Accounts for the cost of loading receiver code and method dispatch.
     pub(crate) send_invoke_method: i64,
 
-    /// Gas cost (Base + len*PerByte) for any Get operation to the IPLD store
-    /// in the runtime VM context.
-    pub(crate) ipld_get_base: i64,
-
-    /// Gas cost (Base + len*PerByte) for any Put operation to the IPLD store
-    /// in the runtime VM context.
-    /// Note: these costs should be significantly higher than the costs for Get
-    /// operations, since they reflect not only serialization/deserialization
-    /// but also persistent storage of chain data.
-    pub(crate) ipld_put_base: i64,
-    pub(crate) ipld_put_per_byte: i64,
-
     /// Gas cost for creating a new actor (via InitActor's Exec method).
     /// Note: this costs assume that the extra will be partially or totally refunded while
     /// the base is covering for the put.
@@ -241,9 +364,23 @@ pub struct PriceList {
     pub(crate) verify_aggregate_seal_steps: AHashMap<RegisteredSealProof, StepCost>,
 
     pub(crate) verify_post_lookup: AHashMap<RegisteredPoStProof, ScalingCost>,
-    pub(crate) verify_post_discount: bool,
     pub(crate) verify_consensus_fault: i64,
     pub(crate) verify_replica_update: i64,
+    // 1 Exec Unit = gas_per_exec_unit * 1 Gas
+    pub(crate) gas_per_exec_unit: i64,
+
+    pub(crate) get_randomness_base: i64,
+    pub(crate) get_randomness_per_byte: i64,
+
+    pub(crate) block_memcpy_per_byte_cost: i64,
+    pub(crate) block_io_per_byte_cost: i64,
+    pub(crate) block_link_per_byte_cost: i64,
+
+    pub(crate) block_open_base: i64,
+    pub(crate) block_read_base: i64,
+    pub(crate) block_create_base: i64,
+    pub(crate) block_link_base: i64,
+    pub(crate) block_stat: i64,
 }
 
 impl PriceList {
@@ -286,20 +423,6 @@ impl PriceList {
         }
         GasCharge::new("OnMethodInvocation", ret, 0)
     }
-    /// Returns the gas required for storing an object.
-    #[inline]
-    pub fn on_ipld_get(&self) -> GasCharge<'static> {
-        GasCharge::new("OnIpldGet", self.ipld_get_base, 0)
-    }
-    /// Returns the gas required for storing an object.
-    #[inline]
-    pub fn on_ipld_put(&self, data_size: usize) -> GasCharge<'static> {
-        GasCharge::new(
-            "OnIpldPut",
-            self.ipld_put_base,
-            data_size as i64 * self.ipld_put_per_byte * self.storage_gas_multiplier,
-        )
-    }
     /// Returns the gas required for creating an actor.
     #[inline]
     pub fn on_create_actor(&self) -> GasCharge<'static> {
@@ -404,10 +527,7 @@ impl PriceList {
                 .expect("512MiB lookup must exist in price table")
         });
 
-        let mut gas_used = cost.flat + info.challenged_sectors.len() as i64 * cost.scale;
-        if self.verify_post_discount {
-            gas_used /= 2;
-        }
+        let gas_used = cost.flat + info.challenged_sectors.len() as i64 * cost.scale;
 
         GasCharge::new("OnVerifyPost", gas_used, 0)
     }
@@ -416,9 +536,115 @@ impl PriceList {
     pub fn on_verify_consensus_fault(&self) -> GasCharge<'static> {
         GasCharge::new("OnVerifyConsensusFault", self.verify_consensus_fault, 0)
     }
+
+    /// Returns the gas required for the specified exec_units.
+    #[inline]
+    pub fn on_consume_exec_units(&self, exec_units: u64) -> GasCharge<'static> {
+        GasCharge::new(
+            "OnConsumeExecUnits",
+            self.gas_per_exec_unit
+                .saturating_mul(i64::try_from(exec_units).unwrap_or(i64::MAX)),
+            0,
+        )
+    }
+
+    /// Converts the specified gas into equivalent exec_units
+    /// Note: In rare cases the provided `gas` may be negative
+    #[inline]
+    pub fn gas_to_exec_units(&self, gas: i64, round_up: bool) -> i64 {
+        match self.gas_per_exec_unit {
+            0 => 0,
+            v => {
+                let mut div_result = gas / v;
+                if round_up && gas % v != 0 {
+                    div_result = div_result.saturating_add(1);
+                }
+                div_result
+            }
+        }
+    }
+
+    /// Returns the cost of the gas required for getting randomness from the client, based on the
+    /// numebr of bytes of entropy.
+    #[inline]
+    pub fn on_get_randomness(&self, entropy_size: usize) -> GasCharge<'static> {
+        GasCharge::new(
+            "OnGetRandomness",
+            self.get_randomness_base.saturating_add(
+                self.get_randomness_per_byte
+                    .saturating_mul(entropy_size as i64),
+            ),
+            0,
+        )
+    }
+
+    /// Returns the base gas required for loading an object, independent of the object's size.
+    #[inline]
+    pub fn on_block_open_base(&self) -> GasCharge<'static> {
+        GasCharge::new("OnBlockOpenBase", self.block_open_base, 0)
+    }
+
+    /// Returns the gas required for loading an object based on the size of the object.
+    #[inline]
+    pub fn on_block_open_per_byte(&self, data_size: usize) -> GasCharge<'static> {
+        // TODO: Should we also throw on a memcpy cost here (see https://github.com/filecoin-project/FIPs/blob/master/FIPS/fip-0032.md#ipld-state-management-fees)
+        GasCharge::new(
+            "OnBlockOpenPerByte",
+            self.block_io_per_byte_cost.saturating_mul(data_size as i64),
+            0,
+        )
+    }
+    /// Returns the gas required for reading a loaded object.
+    #[inline]
+    pub fn on_block_read(&self, data_size: usize) -> GasCharge<'static> {
+        GasCharge::new(
+            "OnBlockRead",
+            self.block_read_base.saturating_add(
+                self.block_memcpy_per_byte_cost
+                    .saturating_mul(data_size as i64),
+            ),
+            0,
+        )
+    }
+
+    /// Returns the gas required for adding an object to the FVM cache.
+    #[inline]
+    pub fn on_block_create(&self, data_size: usize) -> GasCharge<'static> {
+        GasCharge::new(
+            "OnBlockCreate",
+            self.block_create_base.saturating_add(
+                self.block_memcpy_per_byte_cost
+                    .saturating_mul(data_size as i64),
+            ),
+            0,
+        )
+    }
+
+    /// Returns the gas required for committing an object to the state blockstore.
+    #[inline]
+    pub fn on_block_link(&self, data_size: usize) -> GasCharge<'static> {
+        // TODO: The FIP makes it sound like this would need 2 memcpys, is that what's desired?
+        GasCharge::new(
+            "OnBlockLink",
+            self.block_link_base,
+            // data_size as i64 * self.block_link_per_byte_cost * self.storage_gas_multiplier,
+            self.block_link_per_byte_cost
+                .saturating_mul(self.storage_gas_multiplier)
+                .saturating_mul(data_size as i64),
+        )
+    }
+
+    /// Returns the gas required for storing an object.
+    #[inline]
+    pub fn on_block_stat(&self) -> GasCharge<'static> {
+        GasCharge::new("OnBlockStat", self.block_stat, 0)
+    }
 }
 
 /// Returns gas price list by NetworkVersion for gas consumption.
-pub fn price_list_by_network_version(_: NetworkVersion) -> PriceList {
-    OH_SNAP_PRICES.clone()
+pub fn price_list_by_network_version(network_version: NetworkVersion) -> &'static PriceList {
+    match network_version {
+        NetworkVersion::V14 | NetworkVersion::V15 => &OH_SNAP_PRICES,
+        _ => &SKYR_PRICES,
+    }
 }
